local a = CreateFrame("Frame", "Eyes", UIParent)

local _G = getfenv(0)
local GameTooltip, strsub = GameTooltip, strsub
local CheckInteractDistance, GetInventoryItemLink = CheckInteractDistance, GetInventoryItemLink
local UnitExists, UnitIsPlayer, UnitCanAttack = UnitExists, UnitIsPlayer, UnitCanAttack
local oInspectPaperDollFrame_OnShow, oInspectPaperDollItemSlotButton_Update

local cache = { }
local elap, schedule, text = 0, 0.25, nil
local UpdateAllData, UpdateUnit, UpdateNote, UpdateTitleText

UnitPopupButtons.INSPECT.dist = 0  -- enables "Inspect" option in dropdown
BINDING_HEADER_INSPECT = INSPECT
BINDING_NAME_INSPECT = INSPECT

a:SetScript("OnEvent", function(self, event, a1)
	self[event](self, a1)
end)
a:Hide()
a:RegisterEvent("ADDON_LOADED")
function a:ADDON_LOADED(a1)
	if a1 ~= "Blizzard_InspectUI" then return end
	a:UnregisterEvent("ADDON_LOADED")
	a:SetScript("OnEvent", nil)
	a.ADDON_LOADED = nil
	InspectFrame:RegisterEvent("INSPECT_TALENT_READY")
	InspectFrame:SetScript("OnUpdate", nil)
	
	function UpdateAllData()  -- try to update all of the inspect window data
		InspectPaperDollFrame_OnShow()
		InspectPVPFrame_OnShow()
		InspectFrame_UpdateTalentTab()
		SetPortraitTexture(InspectFramePortrait, InspectFrame.unit)
	end
	function UpdateNote(note, timer)  -- inspect status text
		if note then
			elap, schedule = 0, timer
			text:SetText(note)
			a:Show()
		else
			text:SetText("")
			a:Hide()
		end
	end
	function UpdateUnit(unit)  -- update unit
		if not UnitIsPlayer(unit) then return end
		InspectFrame.unit = unit
		InspectNameText:SetText(UnitPVPName(unit))
		UpdateTitleText(unit, true)
		ShowUIPanel(InspectFrame)
		NotifyInspect(unit)
		UpdateAllData()
		if not UnitCanAttack("player", unit) then
			UpdateNote(not CheckInteractDistance(unit, 1) and "Out of Inspect Range", 0.25)
		else
			UpdateNote("Cannot Inspect PVP Enemy", 2)
		end
	end
	local talenttext = { }
	function UpdateTitleText(unit, reset)
		local gname, gtitle = GetGuildInfo(unit)
		if not reset and GetNumTalentTabs() > 0 then
			wipe(talenttext)
			for i = 1, 3, 1 do
				local name, _, pts = GetTalentTabInfo(i, true)
				if not name then return end
				tinsert(talenttext, pts.." "..strsub(name, 1, 3))
			end
		else
			tinsert(talenttext, "-")
		end
		if gname then
			InspectTitleText:SetFormattedText("%s of <%s>\n%s", gtitle, gname, table.concat(talenttext, "/ "))
		else
			InspectTitleText:SetFormattedText("-\n%s", table.concat(talenttext, "/ "))
		end
		wipe(talenttext)
	end

	InspectFrame_Show = function(unit)
		if not UnitIsPlayer(unit) then return end
		PlaySound("igCharacterInfoOpen")
		UpdateUnit(unit)
	end
	
	InspectFrame_OnEvent = function(self, ev, a1)
		if not InspectFrame:IsShown() then return end
		local unit = InspectFrame.unit
		if ((ev == "PLAYER_TARGET_CHANGED" and unit == "target") or (ev == "PARTY_MEMBERS_CHANGED" and unit ~= "target")) and UnitExists(unit) then
			UpdateUnit(unit)
		elseif ev == "UNIT_PORTRAIT_UPDATE" and unit == a1 then
			SetPortraitTexture(InspectFramePortrait, unit)
			UpdateUnit(unit)
		elseif ev == "UNIT_NAME_UPDATE" and unit == a1 then
			InspectNameText:SetText(UnitPVPName(unit))
		elseif ev == "INSPECT_TALENT_READY" then
			UpdateTitleText(unit)
		end
	end
	
	oInspectPaperDollItemSlotButton_Update = InspectPaperDollItemSlotButton_Update
	InspectPaperDollItemSlotButton_Update = function(self, ...)
		oInspectPaperDollItemSlotButton_Update(self, ...)
		if CheckInteractDistance(InspectFrame.unit, 1) then
			local id = self:GetID()
			cache[id] = GetInventoryItemLink(InspectFrame.unit, id)
		end
	end
	
	InspectPaperDollItemSlotButton_OnEnter = function(self)
		self = self or this
		local unit, id = InspectFrame.unit, self:GetID()
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		if UnitExists(unit) and CheckInteractDistance(unit, 1) and GameTooltip:SetInventoryItem(unit, id) then
		
		elseif cache[id] then  -- retrieved from cache
			GameTooltip:SetHyperlink(cache[id])
		else  -- empty slot
			GameTooltip:SetText((self.checkRelic and UnitHasRelicSlot(unit) and _G.RELICSLOT) or _G[strupper(strsub(self:GetName(), 8))])
		end
		CursorUpdate(self)
	end
	
	local function onclick(self, button)
		local unit, id = InspectFrame.unit, self:GetID()
		if UnitExists(unit) and CheckInteractDistance(unit, 1) then
			HandleModifiedItemClick(GetInventoryItemLink(unit, id))
		elseif cache[id] then  -- retrieved from cache
			HandleModifiedItemClick(cache[id])
		end
	end
	for _, frame in ipairs({ InspectPaperDollFrame:GetChildren() }) do
		if frame:GetFrameType() == "Button" and strmatch(frame:GetName(), "Inspect(.+)Slot") then
			frame:SetScript("OnClick", onclick)
		end
	end
	
	oInspectPaperDollFrame_OnShow = InspectPaperDollFrame_OnShow
	InspectPaperDollFrame_OnShow = function(...)
		if not UnitIsPlayer(InspectFrame.unit) then return end
		oInspectPaperDollFrame_OnShow(...)
	end
	
	InspectNameText:SetWidth(330)
	InspectTitleText:Show()
	InspectTitleText:SetText("")
	text = InspectFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	text:SetPoint("TOP", InspectFrame, "TOP", 0, -1)
	text:SetAlpha(0.8)
	
	a:SetScript("OnUpdate", function(this, a1)
		elap = elap + a1
		if elap < schedule then return end
		elap = 0
		
		local unit = InspectFrame.unit
		if not UnitExists(unit) or schedule == 2 then 
			UpdateNote()
		elseif CheckInteractDistance(unit, 1) then
			NotifyInspect(unit)
			UpdateAllData()
			UpdateNote("Inspect Successful", 2)
		end
	end)
end
